--[[ KlaxxiTargets ]]

local DefaultO = {
  ["blnMarkBosses"] = false;
  ["blnShowFrame"] = false;
  ["framePoint"] = "LEFT";
  ["frameRelativeTo"] = "UIParent";
  ["frameRelativePoint"] = "CENTER";
  ["frameOffsetX"] = 200;
  ["frameOffsetY"] = 0;
}

local isKlaxxiBosses = nil
local markKlaxxiBosses = nil
local isHeroicDifficulty = nil
local markBoss = nil
local UnitGUID = UnitGUID
local UnitExists = UnitExists
local moving = false
local expWod = select(4, GetBuildInfo()) >= 60000

local klaxxiBosses = {
  [71158] = 1; --"Rik'kal the Dissector"
  [71152] = 2; --"Skeer the Bloodseeker"
  [71155] = 3; --"Korven the Prime"
  [71153] = 4; --"Hisek the Swarmkeeper"
  [71154] = 5; --"Ka'roz the Locust"
  [71157] = 6; --"Xaril the Poisoned Mind"
  [71160] = 7; --"Iyyokuk the Lucid"
  [71156] = 8; --"Kaz'tik the Manipulator"
  [71161] = 9; --"Kil'ruk the Wind-Reaver"
};
local klaxxiPriorities = {
  --bossNumber 1, 2, 3, skull, cross, TankABoss1, TankABoss2, TankBBoss1, TankBBoss2, TankForNextBoss
  --[[
    example:
      {1, 2, 4, 1, 2, 1, 4, 2, 0, 2};
    bosses active:
      1, 2 and 4 (Rik'kal, Skeer and Hisek)
    skull target:
      1 (Rik'kal)
    cross target:
      2 (Skeer)
    bosses for TankA:
      1 and 4 (Rik'kal and Hisek)
    bosses for TankB:
      2 (and 0, so only one boss) (Skeer)
    TankForNextBoss:
      2 (TankB will take the next boss)
  --]]
  ["nhc"] = {
    {1, 2, 4, 1, 2, 1, 4, 2, 0, 2};
    {2, 4, 5, 2, 3, 4, 0, 2, 5, 1};
    {3, 4, 5, 3, 4, 3, 4, 5, 0, 2};
    {4, 5, 7, 4, 5, 4, 0, 5, 7, 1};
    {5, 6, 7, 5, 6, 6, 0, 5, 7, 1};
    {6, 7, 8, 6, 7, 6, 8, 7, 0, 2};
    {7, 8, 9, 7, 8, 8, 0, 7, 9, 0};
    {8, 9, 0, 8, 9, 8, 0, 9, 0, 0};
    {9, 0, 0, 9, 0, 0, 0, 9, 0, 0};
  };
  ["hc"] = {
    {1, 2, 4, 2, 1, 1, 0, 2, 0, 2};
    {1, 4, 5, 1, 3, 1, 0, 5, 0, 1};
    {3, 4, 5, 3, 4, 3, 0, 5, 0, 1};
    {4, 5, 7, 4, 6, 7, 0, 5, 0, 1};
    {5, 6, 7, 6, 8, 6, 7, 5, 0, 1};
    {5, 7, 8, 8, 7, 7, 8, 5, 0, 2};
    {5, 7, 9, 7, 5, 5, 7, 9, 0, 0};
    {5, 9, 0, 5, 9, 5, 0, 9, 0, 0};
    {9, 0, 0, 9, 0, 0, 0, 9, 0, 0};
  };
};
local klaxxiTankNotes = {
  [1] = "Rik'kal (Injection, active mitigation)"; --T, spawn Parasites (100% heal every 10sec, 80k dps), not: Skeer
  [2] = "Skeer";                                  --T, spawns Blood-Adds, not: Rik'kal
  [3] = "Korven (Shield Bash)";                   --T, taunt on Shield Bash
  [4] = "Hisek";                                  --Center, Rapid Fire
  [5] = "Ka'roz";                                 --T, flashes around, hurl amber from platforms
  [6] = "Xaril (active mitigation)";              --T, explosion at 10 stacks, not: Kil'ruk, green: 3 players step into fog
  [7] = "Iyyokuk";                                --T, fiery edge
  [8] = "Kaz'tik";                                --kunchongs, sonic projection?
  [9] = "Kil'ruk (Gouge, Reave)";                 --T, gouge, not: Xaril
};

local KlaxxiTargets_UpdateOptionsArray, KlaxxiTargets_CloneTable, KlaxxiTargets_CopyValues
function KlaxxiTargets_UpdateOptionsArray()
  local TempKlaxxiTargetsOptions = KlaxxiTargets_CloneTable(DefaultO);
  KlaxxiTargets_CopyValues(KlaxxiTargetsOptions, TempKlaxxiTargetsOptions);
  KlaxxiTargetsOptions = TempKlaxxiTargetsOptions
end
--from "SCT" by "Grayhoof"
function KlaxxiTargets_CloneTable(t) -- return a copy of the table t
  local new = {}; -- create a new table
  local i, v = next(t, nil); -- i is an index of t, v = t[i]
  while i do
    if type(v)=="table" then
      v=KlaxxiTargets_CloneTable(v);
    end
    new[i] = v;
    i, v = next(t, i); -- get next index
  end
  return new;
end
function KlaxxiTargets_CopyValues(from,to)
  local i, v = next(from, nil); -- i is an index of from, v = from[i]
  while i do
    if type(v)=="table" then
      if to[i] ~= nil then
        if type(to[i])=="table" then
          KlaxxiTargets_CopyValues(v,to[i]);
        end
      end
    else
      if to[i] ~= nil then
        if type(to[i])~="table" then
          to[i] = v;
        end
      end
    end
    i, v = next(from, i); -- get next index
  end
end

local function getNPCID(unit)
  if expWod then
    if (guid:sub(1, 9)) == "Creature:" then
      local npcid = string.match(guid, ".-:.-:.-:.-:.-:(%d+):.+") --Creature:0:[serverID]:[instanceID]:[zoneUID]:[npcID]:[spawnUID]
      return npcid
    end
    return 0
  else
    return tonumber((UnitGUID(unit)):sub(6, 10), 16)
  end
end

local frame = CreateFrame("Frame", "KlaxxiTargetsFrame", UIParent)
local frameEvents = {};

local function initFrame()
  --frame position
  frame:SetPoint(KlaxxiTargetsOptions["framePoint"], KlaxxiTargetsOptions["frameRelativeTo"], KlaxxiTargetsOptions["frameRelativePoint"], KlaxxiTargetsOptions["frameOffsetX"], KlaxxiTargetsOptions["frameOffsetY"])
  frame:SetFrameStrata("LOW")
  frame:SetSize(250, 66)
  frame:Show()
  
  frame.bgtexture = frame:CreateTexture(nil, "BACKGROUND")
  frame.bgtexture:SetAllPoints(frame)
  frame.bgtexture:SetTexture(0, 0, 0, 0.8)
  frame.bgtexture:Hide()
  frame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
  frame:SetScript("OnDragStop", function(self)
    self:StopMovingOrSizing()
    local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()
    KlaxxiTargetsOptions["framePoint"] = point or "LEFT"
    KlaxxiTargetsOptions["frameRelativeTo"] = relativeTo or "UIParent"
    KlaxxiTargetsOptions["frameRelativePoint"] = relativePoint or "CENTER"
    KlaxxiTargetsOptions["frameOffsetX"] = xOfs
    KlaxxiTargetsOptions["frameOffsetY"] = yOfs
  end);
  
  local tankAcolor, tankBcolor = RAID_CLASS_COLORS["DRUID"], RAID_CLASS_COLORS["DEATHKNIGHT"]
  frame.tankAtexts = {};
  frame.tankBtexts = {};
  
  for i = 1, 3 do
    frame.tankAtexts[i] = frame:CreateFontString(nil, "LOW")
    frame.tankAtexts[i]:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE")
    frame.tankAtexts[i]:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, -11 * (i-1))
    frame.tankAtexts[i]:SetText("")
    frame.tankAtexts[i]:SetTextColor(tankAcolor.r, tankAcolor.g, tankAcolor.b, 1)
  end
  
  for i = 1, 3 do
    frame.tankBtexts[i] = frame:CreateFontString(nil, "LOW")
    frame.tankBtexts[i]:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE")
    frame.tankBtexts[i]:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, -11 * (i+2))
    frame.tankBtexts[i]:SetText("")
    frame.tankBtexts[i]:SetTextColor(tankBcolor.r, tankBcolor.g, tankBcolor.b, 1)
  end
  
  frame.clearTankText = function(self)
    for i = 1, 3 do
      self.tankAtexts[i]:SetText("")
      self.tankBtexts[i]:SetText("")
    end
  end
  frame.setTankText = function(self, aTank, nText, text)
    if aTank == "A" then
      self.tankAtexts[nText]:SetText(text)
    elseif aTank == "B" then
      self.tankBtexts[nText]:SetText(text)
    end
  end
  frame.rt7Text = "\124TInterface\\TargetingFrame\\UI-RaidTargetingIcon_7:0\124t "
  frame.rt8Text = "\124TInterface\\TargetingFrame\\UI-RaidTargetingIcon_8:0\124t "
end

isKlaxxiBosses = function()
  for i = 1, MAX_BOSS_FRAMES do
    if UnitExists("boss"..i) then
      local bossNPCid = getNPCID("boss"..i)
      for id, n in pairs(klaxxiBosses) do
        if bossNPCid == id then
          return true
        end
      end
    end
  end
  return false
end

markBoss = function(iBoss, iconId)
  if KlaxxiTargetsOptions["blnMarkBosses"] and GetRaidTargetIndex("boss"..iBoss) ~= iconId then
    SetRaidTarget("boss"..iBoss, iconId)
  end
end

isHeroicDifficulty = function()
  --local name, instanceType, difficultyID, difficultyName, maxPlayers, dynamicDifficulty, isDynamic, instanceMapID, instanceGroupSize = GetInstanceInfo()
  local _, _, difficultyID = GetInstanceInfo()
  
  local isHeroic
  if expWod then
    --New: name, groupType, isHeroic, isChallengeMode, displayHeroic, displayMythic, toggleDifficultyID = GetDifficultyInfo(id)
    _, _, _, _, _, isHeroic = GetDifficultyInfo(difficultyID)
  else
    --local dif_name, groupType, isHeroic, isChallengeMode, toggleDifficultyID = GetDifficultyInfo(difficultyID) --http://wowpedia.org/API_GetDifficultyInfo
    _, _, isHeroic = GetDifficultyInfo(difficultyID)
  end
  return isHeroic
end

markKlaxxiBosses = function()
  local prios
  if isHeroicDifficulty() then
    prios = klaxxiPriorities["hc"]
  else
    prios = klaxxiPriorities["nhc"]
  end
  local row, i, numActiveBosses, skullTarget, crossTarget = 0, 1, 0, 0, 0
  local ab, prioRow = 0, nil
  local nBoss1, nBoss2, nBoss3, nBoss4 = -1, -1, -1, -1
  local id
  
  if UnitExists("boss1") then
    numActiveBosses = 1
    id = getNPCID("boss1")
    nBoss1 = klaxxiBosses[id] or -1
  end
  if UnitExists("boss2") then
    numActiveBosses = 2
    id = getNPCID("boss2")
    nBoss2 = klaxxiBosses[id] or -1
  end
  if UnitExists("boss3") then
    numActiveBosses = 3
    id = getNPCID("boss3")
    nBoss3 = klaxxiBosses[id] or -1
  end
  if UnitExists("boss4") then
    id = getNPCID("boss4")
    nBoss4 = klaxxiBosses[id] or -1
  end
  
  while (row == 0) and (i <= #prios) do
    ab = 0
    prioRow = prios[i]
    for iRow = 1, numActiveBosses do
      if prioRow[iRow] == nBoss1 or prioRow[iRow] == nBoss2 or prioRow[iRow] == nBoss3 then
        ab = ab + 1
      end
    end
    if ab == numActiveBosses then
      row = i
    end
    i = i + 1
  end
  
  if row == 0 then
    --you dun messed up, a.a.ron
    frame:clearTankText()
  else
    local prio = prios[row]
    skullTarget = prio[4]
    if skullTarget == 0 then
      skullTarget = -1
    end
    crossTarget = prio[5]
    if crossTarget == 0 then
      crossTarget = -1
    end
    
    if nBoss1 == skullTarget then
      markBoss(1, 8)
    elseif nBoss2 == skullTarget then
      markBoss(2, 8)
    elseif nBoss3 == skullTarget then
      markBoss(3, 8)
    end
    
    if nBoss1 == crossTarget then
      markBoss(1, 7)
    elseif nBoss2 == crossTarget then
      markBoss(2, 7)
    elseif nBoss3 == crossTarget then
      markBoss(3, 7)
    elseif nBoss4 == crossTarget then --also mark bosses not yet active
      markBoss(4, 7)
    end
    
    local firstTankPos = 6
    
    frame:setTankText("A", 1, format("%s%s", prio[firstTankPos] == crossTarget and frame.rt7Text or prio[firstTankPos] == skullTarget and frame.rt8Text or "", klaxxiTankNotes[prio[firstTankPos]] or ""))
    frame:setTankText("A", 2, format("%s%s", prio[firstTankPos+1] == crossTarget and frame.rt7Text or prio[firstTankPos+1] == skullTarget and frame.rt8Text or "", klaxxiTankNotes[prio[firstTankPos+1]] or ""))
    frame:setTankText("A", 3, format("%s%s", nBoss4 == crossTarget and prio[firstTankPos + 4] == 1 and frame.rt7Text or "", prio[firstTankPos + 4] == 1 and "(next)" or ""))
    
    frame:setTankText("B", 1, format("%s%s", prio[firstTankPos+2] == crossTarget and frame.rt7Text or prio[firstTankPos+2] == skullTarget and frame.rt8Text or "", klaxxiTankNotes[prio[firstTankPos+2]] or ""))
    frame:setTankText("B", 2, format("%s%s", prio[firstTankPos+3] == crossTarget and frame.rt7Text or prio[firstTankPos+3] == skullTarget and frame.rt8Text or "", klaxxiTankNotes[prio[firstTankPos+3]] or ""))
    frame:setTankText("B", 3, format("%s%s", nBoss4 == crossTarget and prio[firstTankPos + 4] == 2 and frame.rt7Text or "", prio[firstTankPos + 4] == 2 and "(next)" or ""))
  end
end

function frameEvents:PLAYER_ENTERING_WORLD(...)
  frame:UnregisterEvent("PLAYER_ENTERING_WORLD")
  
  if not KlaxxiTargetsOptions then
    KlaxxiTargetsOptions = DefaultO
  end
  KlaxxiTargets_UpdateOptionsArray()
  initFrame()
  if KlaxxiTargetsOptions["blnShowFrame"] then
    frame:Show()
  else
    frame:Hide()
  end
  
  frame:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")
end
function frameEvents:INSTANCE_ENCOUNTER_ENGAGE_UNIT()
  if isKlaxxiBosses() then
    markKlaxxiBosses()
  else
    self:clearTankText()
  end
end
frame:SetScript("OnEvent", function(self, event, ...)
  frameEvents[event](self, ...)
end);
frame:RegisterEvent("PLAYER_ENTERING_WORLD")

local function mysplit(inputstr, sep)
  if sep == nil then
    sep = "%s"
  end
  local t = {}
  local i = 1
  for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
    t[i] = str
    i = i + 1
  end
  return t
end

SLASH_KLAXXITARGETS1 = "/klaxxi" --SLASH_KLAXXITARGETS2 = "/klaxxitargets"
SlashCmdList["KLAXXITARGETS"] = function(msg, editbox)
  msg = msg or ""
  args = mysplit(msg, " ")
  
  if string.lower(args[1] or "") == "mark" then
    if KlaxxiTargetsOptions["blnMarkBosses"] == true then
      KlaxxiTargetsOptions["blnMarkBosses"] = false
    else
      KlaxxiTargetsOptions["blnMarkBosses"] = true
    end
    print("|cffaaaaffKlaxxiTargets |rmark |cffaaaaffis now "..(KlaxxiTargetsOptions["blnMarkBosses"] == true and "|cffaaffaaON" or "|cffff8888OFF"))
  elseif string.lower(args[1] or "") == "frame" then
    if KlaxxiTargetsOptions["blnShowFrame"] == true then
      KlaxxiTargetsOptions["blnShowFrame"] = false
      frame:Hide()
    else
      KlaxxiTargetsOptions["blnShowFrame"] = true
      frame:Show()
    end
    print("|cffaaaaffKlaxxiTargets |rframe |cffaaaaffis now "..(KlaxxiTargetsOptions["blnShowFrame"] == true and "|cffaaffaaON" or "|cffff8888OFF"))
  elseif string.lower(args[1] or "") == "move" then
    if moving then
      moving = false
      frame:SetMovable(false) --click-through
      frame:EnableMouse(false)
      frame:RegisterForDrag("")
      frame.bgtexture:Hide()
      
      frame:clearTankText()
    else
      moving = true
      frame:SetMovable(true);
      frame:EnableMouse(true);
      frame:RegisterForDrag("LeftButton");
      frame.bgtexture:Show()
      
      frame:setTankText("A", 1, format("%s%s", frame.rt8Text, "TankA first boss, also skull target"))
      frame:setTankText("A", 2, format("%s%s", "",            "TankA second boss"))
      frame:setTankText("A", 3, format("%s%s", "", ""))
      
      frame:setTankText("B", 1, format("%s%s", frame.rt7Text, "TankB first boss, also cross"))
      frame:setTankText("B", 2, format("%s%s", "", ""))
      frame:setTankText("B", 3, format("%s%s", "", "(next) <-- the next boss is for TankB"))
    end
    print("|cffaaaaffKlaxxiTargets |rmove |cffaaaaffis now "..(moving == true and "|cffaaffaamoving" or "|cffff8888fixed"))
  elseif string.lower(args[1] or "") == "reset" then
    KlaxxiTargetsOptions["framePoint"] = DefaultO["framePoint"]
    KlaxxiTargetsOptions["frameRelativeTo"] = DefaultO["frameRelativeTo"]
    KlaxxiTargetsOptions["frameRelativePoint"] = DefaultO["frameRelativePoint"]
    KlaxxiTargetsOptions["frameOffsetX"] = DefaultO["frameOffsetX"]
    KlaxxiTargetsOptions["frameOffsetY"] = DefaultO["frameOffsetY"]
    
    frame:ClearAllPoints()
    frame:SetPoint(KlaxxiTargetsOptions["framePoint"], KlaxxiTargetsOptions["frameRelativeTo"], KlaxxiTargetsOptions["frameRelativePoint"], KlaxxiTargetsOptions["frameOffsetX"], KlaxxiTargetsOptions["frameOffsetY"])
    
    print("|cffaaaaffKlaxxiTargets |rposition reset")
  else
    print("|cffaaaaffKlaxxiTargets "..(GetAddOnMetadata("KlaxxiTargets", "Version") or ""))
    print("|cffaaaaff  use |r/klaxxi <option> |cffaaaaffto toggle one of these options:")
    print("  mark |cffaaaaffset raid markers on klaxxi bosses (skull and cross) ("..(KlaxxiTargetsOptions["blnMarkBosses"] == true and "|cffaaffaaON" or "|cffff8888OFF").."|cffaaaaff)")
    print("  frame |cffaaaaffshow/hide a frame showing all active bosses ("..(KlaxxiTargetsOptions["blnShowFrame"] == true and "|cffaaffaaON" or "|cffff8888OFF").."|cffaaaaff)")
    print("  move |cffaaaafftoggle moving the frame ("..(moving == true and "|cffaaffaamoving" or "|cffff8888fixed").."|cffaaaaff)")
    print("  reset |cffaaaaffreset the frame's position")
  end
end
